//
//  PlayingCard.swift
//  PlayingCard
//
//  Created by Sameh Fakhouri on 10/2/19.
//  Copyright © 2019 Sameh Fakhouri. All rights reserved.
//

import Foundation

struct PlayingCard: CustomStringConvertible {
    
    var description: String {
        return "\(rank)\(suit)"
    }
    
    
    var suit : Suit
    var rank : Rank
    
    
    enum Suit: String, CustomStringConvertible {
        var description: String {
            return self.rawValue
        }
        
        case clubs = "♣️"
        case diamonds = "♦️"
        case hearts = "❤️"
        case spades = "♠️"
        
        static var all = [Suit.clubs, .diamonds, .hearts, .spades]
    }
 
    
    
    
    
    enum Rank: CustomStringConvertible {
        
        var description: String {
            switch self {
            case .ace: return "A"
            case .numeric(let num): return(String(num))
            case .face(let kind): return kind
            }
        }
        
        case ace
        case face(String)
        case numeric(Int)
        
        var order: Int {
            switch self {
            case .ace: return 1
            case .numeric(let num): return num
            case .face(let kind) where kind == "J": return 11
            case .face(let kind) where kind == "Q": return 12
            case .face(let kind) where kind == "K": return 13
            default:
                return 0
            }
        }
        
        static var all: [Rank] {
            var allRanks = [Rank.ace]
            for num in 2...10 {
                allRanks.append(.numeric(num))
            }
            allRanks += [.face("J"), .face("Q"), .face("K")]
            return allRanks
        }
    }
}
